/* 
 * This is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This software is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this software; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,
 * USA.
 */
package android.androidVNC;

import java.io.File;
import java.io.FileOutputStream;
import java.util.Calendar;
import java.util.GregorianCalendar;

import android.app.AlertDialog;
import android.graphics.Bitmap;
import android.os.Environment;
import android.widget.Toast;
/**
 * <p>(Se omiten acentos)</p>
 * <p>Clase utilizada para realizar una captura de pantalla local.</p> 
 * <p>Screenshot.java.</p>
 * <p>Aadido al PFC en la version v0.1.5.</p>
 * <p><b>Proyecto original:</b></p>
 * 		<p>Web: http://code.google.com/p/android-vnc-viewer/.</p>
 * 		<p>Release: 39.</p>
 *	 	<p>Migrada a release 60.</p>
 * @author Alberto Poblacion Navarro.
 * 
 */
public class Screenshot {
	/**
	 * Realiza un screenshot del contenido actual mostrado en pantalla
	 * @param vncCanvas Activity Instancia de esta clase utilizada. Maneja el lienzo.
	 * @param vncCanvas Instancia del lienzo donde se dibuja la imagen.
	 * @return True Si se realiza el screenshot correctamente, false en caso contrario
	 */
	static public boolean nuevo(VncCanvasActivity vncCanvasActivity, VncCanvas vncCanvas){
		
		int num_imagen=0;
		boolean retorno = true;
		//Se obtiene la instancia del zoom para poder aplicarlo a la imagen.
		Zoom zoom = Zoom.Instancia(vncCanvasActivity, vncCanvas);
		String path=null;
		String sdcard_status = Environment.getExternalStorageState();
		File carpeta;
		//Se comprueba si hay un medio extraible donde se pueda guardar la imagen
		if(sdcard_status.equals(Environment.MEDIA_MOUNTED)){
        	path = "/sdcard/screenshot/"; //Si hay tarjeta se guarda en la tarjeta
		}else{
			//Si no hay tarjeta de memoria se crea la carpeta en la memoria interna del movil
        	//Se crea en la carpeta del propio programa
        	path = "/data/data/android.androidVNC/screenshot/";
        } 
        carpeta=new File(path);
        if(!carpeta.exists())
		  carpeta.mkdir(); //Si la carpeta no existe se crea
		 int dia=0;
		 int mes=0;
		 int year=0;
		 try{
			  dia = new GregorianCalendar().get(Calendar.DAY_OF_MONTH);
			  mes = (new GregorianCalendar().get(Calendar.MONTH))+1; //No obtiene bien el mes
			  year = new GregorianCalendar().get(Calendar.YEAR);
		 }catch (Exception e) {
			if (e instanceof IllegalArgumentException){
				Toast.makeText(vncCanvasActivity, vncCanvasActivity.getString(R.string.no_date), Toast.LENGTH_SHORT).show();
				dia=mes=year=0;

			}
		 }
		 String nombre = new String("screenshot"+dia+mes+year+".jpeg");
		  //Archivo que usara el buffer de salida para guardar la imagen
		  File imagen = new File(path+nombre); 
		  while(imagen.exists())
		  {
			  nombre= new String("screenshot"+dia+mes+year+"("+num_imagen+")"+".jpeg");
			  imagen= new File(path+nombre);
			  num_imagen++;
		  }
		  num_imagen=0;
		  Bitmap nueva=null;
		  //Coje solo lo se muestra en la pantalla del movil aplicando el mismo zoom que se est aplicando
		  try{
			  nueva = Bitmap.createBitmap(vncCanvas.bitmapData.mbitmap,
				 		(int)(vncCanvasActivity.absoluteXPosition/zoom.getZoom()),
				 		(int)(vncCanvasActivity.absoluteYPosition/zoom.getZoom()),
						(int)(vncCanvas.getWidth()/zoom.getZoom()), 
						(int)(vncCanvas.getHeight()/zoom.getZoom()), 
						Zoom.getMatrix(), true);
		  }catch (Exception e) {
			  new AlertDialog.Builder(vncCanvasActivity)
	          .setTitle(vncCanvasActivity.getString(R.string.error))
	          .setMessage(vncCanvasActivity.getString(R.string.cause)+": "+e.toString())
	          .setIcon(android.R.drawable.ic_dialog_alert)
	          .setPositiveButton(android.R.string.ok, null)
	          .show();
			retorno=false;
		  }
		  //Guarda la imagen en formato JPEG y se muestra un mensaje confirmandolo
		  try{
			  FileOutputStream fos= new FileOutputStream(imagen);
			  nueva.compress(Bitmap.CompressFormat.JPEG,100,fos);
			  fos.flush();
			  fos.close();
			  Toast.makeText(vncCanvasActivity, vncCanvasActivity.getString(R.string.screenshot)+": "+ nombre, Toast.LENGTH_SHORT).show();
		  }catch (Exception e) {
			  //En caso de error se muestra un cuadro de dialogo con el error
			  	  new AlertDialog.Builder(vncCanvasActivity)
		          .setTitle(vncCanvasActivity.getString(R.string.error))
		          .setMessage(vncCanvasActivity.getString(R.string.cause)+": "+e.toString())
		          .setIcon(android.R.drawable.ic_dialog_alert)
		          .setPositiveButton(android.R.string.ok, null)
		          .show();
				retorno=false;
			}finally{
				nueva.recycle(); //Una vez guardada la imagen se libera el recurso.
			}
		  if(retorno)
			  Notificacion.Instancia(vncCanvasActivity).notificar(vncCanvasActivity.getString(R.string.screenshot),vncCanvasActivity.getString(R.string.image_saved)+": "+path+nombre, imagen);
		  return retorno;
	}
}
