/* 
 * This is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This software is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this software; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,
 * USA.
 */
package android.androidVNC;

import android.app.AlertDialog;
import android.content.DialogInterface;
import android.graphics.Matrix;
import android.widget.ImageView.ScaleType;
/**
 * <p>(Se omiten acentos)</p>
 * <p>Clase que permite escalar la imagen del escritorio local para facilitar su comprension.</p>
 * <p>Fichero: Zoom.java</p>
 * <p>Aadido al PFC en la version v0.1.8</p>
 * <p><b>Proyecto original:</b></p>
 * 		<p>Web: http://code.google.com/p/android-vnc-viewer/.</p>
 * 		<p>Release: 39.</p>
 *	 	<p>Migrada a release 60.</p>
 * @author Alberto Poblacion Navarro</p>
 */
public class Zoom {
	/** Cadena para identificar la clase en el log interno*/
	private static final String TAG = "ZOOM";
	/** Variable con la escala que se va a realizar */
	private float zoom =1; 			
	/** Cantidad que se va a aadir a la escala */
	private final float variacion=0.1f;
	/** Flag para conocer si es la primera ejecucion del zoom */
	private boolean first_time=true;
	/** Variable que almacena el ancho inicial de la imagen */
	private int ancho=0; 			
	/** Variable que almacena el alto inicial de la imagen*/
 	private int alto=0; 			
 	/** Variable que almacena la posicion del panning en el eje x*/
	private int absoluteX=0;		
	/** Variable que almacena la posicion del panning en el eje y*/
	private int absoluteY=0;		
	/** Desplazamiento inicial sobre la imagen en  el eje x */
	private int scrollXini=0;		
	/** Desplazamiento inicial sobre la imagen en  el eje y */
	private int scrollYini=0;		
	/** Matriz utilizada en el escalado de la imagen*/
	private static Matrix matriz; 
	/** Zoom out maximo permitido */
	final float MIN_ZOOM=0.6f; 	
	/** Zoom in maximo permitido */
	final float MAX_ZOOM=2.6f; 	
	/** Tipo de zoom: zoom in */
	final int IN=0; 
	/** Tipo de zoom: zoom out */
	final int OUT=1;
	/** Instancia de la clase encargada de manejar los eventos de entrada */
	private VncCanvasActivity vncCanvasActivity;
	/** Instancia de la clase encargada de dibujar la imagen */
	private VncCanvas vncCanvas;
	/** Objeto que almacena la unica instancia que debe crearse de esta clase en ejecucion. Patron Singleton */
	private static Zoom instancia = null;
	/**
	 * Constructor publico de la clase. Solo se va a crear una instancia por ejecucion del programa.
	 * En caso de que no se haya creado la instancia se crea, en caso contrario se devuelve la creada anteriormente.
	 * @return instancia de la clase.
	 */
	static synchronized Zoom Instancia(VncCanvasActivity vnccanvasactivity, VncCanvas vnccanvas){
		if (instancia == null)
			instancia = new Zoom(vnccanvas, vnccanvasactivity);
		return instancia;
	}
	/**
	 * Constructor de la clase donde se iniciliazan las variables
	 * @param vnccanvas Instancia de la clase encargada de dibujar la imagen
	 * @param vnccanvasactivity Instancia de la clase encargada de manejar los eventos de entrada
	 */
	private Zoom(VncCanvas vnccanvas, VncCanvasActivity vnccanvasactivity) {
		vncCanvas=vnccanvas;
		vncCanvasActivity=vnccanvasactivity;
		matriz = new Matrix();
	}
	/**
	 * Guarda la informacion del lienzo y la imagen antes de realizarle un zoom
	 * @param vnccanvas Lienzo al cual se va a realizar un zoom
	 */
	private void guardar_datos_iniciales(){
		if(first_time){
			//Se guarda las dimensiones de la imagen
			ancho=vncCanvas.bitmapData.framebufferwidth;
			alto=vncCanvas.bitmapData.framebufferheight;
			//Se guarda el desplazamiento de la imagen en ScaleType.CENTER
			scrollXini=vncCanvas.getScrollX();
			scrollYini=vncCanvas.getScrollY();
			//Se guarda la posicion absoluta inicial de la esquina superior izquierda
			absoluteX=vncCanvasActivity.absoluteXPosition;
			absoluteY=vncCanvasActivity.absoluteYPosition;
			//Cantidad de zoom
			zoom=1;
			first_time=false;
		}
	}
	/**	
	 * Realiza un "zoom in" a la imagen centrada en el raton, llamando a la funcion zoom con el tipo IN
	 * @param vncCanvas Lienzo al cual se va a realizar un zoom
	 * @param vncCanvasActivity Actividad que se esta ejecutando y que muestra el lienzo
	 * @return True si se realiza correctamente, false en caso contrario
	 */
	public boolean zoom_in(){
		return zoom(vncCanvas, vncCanvasActivity, IN);	
	}
	/**
	 * Realiza un "zoom out" a la imagen centrada en el raton, llamando a la funcion zoom con el tipo IN
	 * @param vncCanvas Lienzo al cual se va a realizar un zoom
	 * @param vncCanvasActivity Actividad que se esta ejecutando y que muestra el lienzo
	 * @return True si se realiza correctamente, false en caso contrario
	 */
	public boolean zoom_out(){
		return zoom(vncCanvas, vncCanvasActivity, OUT);
	}
	/**
     * Redondea un valor double a un numero especifico de decimales
     * @param val el valor que se va a redondear.
     * @param places el numero de decimales a redondear.
     * @return val valor redeando a los decimales.
     */
	public static double round(double val, int places) {
		 long factor = (long)Math.pow(10,places);
		 // Cambia el decimal el numero de posiciones necesarias a la derecha
		 val = val * factor;
		 // Redondea al entero mas cercano
		 long tmp = Math.round(val);
		 // Cambia el decimal al numero de posiciones a la izquierda
		 return (double)tmp / factor;
	}
	/**
	 * Realiza el zoom a la imagen centrada en raton del tipo indicada en el parametro tipo
	 * @param vncCanvas Lienzo al cual se va a realizar un zoom
	 * @param vncCanvasActivity Actividad que se esta ejecutando y que muestra el lienzo
	 * @param tipo Tipo de zoom: zoom in o zoom out
	 * @return true si se realiza el zoom correctamente, false en caso contrario
	 * @author Alberto Poblacion Navarro
	 */
	private boolean zoom(VncCanvas vncCanvas, VncCanvasActivity vncCanvasActivity, int tipo){
		Notificacion notificacion = Notificacion.Instancia(vncCanvasActivity);
		//Guard los valores iniciales de la imagen para poder restaurarla.
		guardar_datos_iniciales();
		//Se aade valores al zoom necesarios segun el tipo
		if(tipo==IN) 
			zoom+=variacion;
		else //type == OUT 
			zoom-=variacion;
		zoom=(float)round((double)zoom,2);
				
		
		if((zoom < MAX_ZOOM)&&(zoom > MIN_ZOOM)){
			notificacion.notificar("Zoom realizado", "Zoom establecido en "+zoom*100+"%");
			AndroidLog.Instancia().log(TAG, "Zoom establecido en "+zoom+"%");
			//Se configura la escala que se va a aplicar a la imagen
			matriz.setScale(zoom ,zoom);
			//Se establece la matriz con la escala al lienzo
			vncCanvas.setImageMatrix(matriz);
			//Se establece el tipo de escalado a matriz (si no se hace el cambio no se actualiza)
			vncCanvas.setScaleType(ScaleType.CENTER);
			vncCanvas.setScaleType(ScaleType.MATRIX);
			//Dado que el tamao de la imagen va a cambiar, se cambian las variables que guardan dicho valor
			vncCanvas.bitmapData.framebufferheight= (int)(alto*zoom); 
			vncCanvas.bitmapData.framebufferwidth= (int)(ancho*zoom);
			//Se centra el zoom en la posicion del raton
			vncCanvasActivity.absoluteXPosition=(int)(vncCanvas.mouseX*zoom-vncCanvas.getWidth()/2);
			vncCanvasActivity.absoluteYPosition=(int)(vncCanvas.mouseY*zoom-vncCanvas.getHeight()/2);
			//Desplazamiento que se va a realizar en la imagen
			int scrollX=vncCanvasActivity.absoluteXPosition;
			int scrollY=vncCanvasActivity.absoluteYPosition;
		
			//Si se superan los limites de la imagen el zoom se centra en la esquina sin salir de la imagen
			//Limites izquierdo y derecho
			if(scrollX<0){
				vncCanvasActivity.absoluteXPosition=scrollX=0;
				
			}else if ((vncCanvasActivity.absoluteXPosition+vncCanvas.getWidth())>vncCanvas.bitmapData.framebufferwidth){
				vncCanvasActivity.absoluteXPosition= scrollX=vncCanvas.bitmapData.framebufferwidth-vncCanvas.getWidth();
			}
			//Limites superior e inferior
			if(scrollY<0){
				vncCanvasActivity.absoluteYPosition=scrollY=0;
			}
			else if ((vncCanvasActivity.absoluteYPosition+vncCanvas.getHeight())>vncCanvas.bitmapData.framebufferheight){
				vncCanvasActivity.absoluteYPosition=scrollY=vncCanvas.bitmapData.framebufferheight-vncCanvas.getHeight()-vncCanvas.getTop();
			}

			//Se desplaza la imagen a la posicion indicada.			
			vncCanvas.scrollTo(scrollX,scrollY);
			return true;
			
		}else{
			//Si se ha alcanzado los limites del zoom, se actualiza la variable y se avisa
			if(tipo==IN) zoom -= variacion;
			else zoom += variacion;
			
			new AlertDialog.Builder(vncCanvasActivity)
	          .setTitle(vncCanvasActivity.getString(R.string.max_zoom))
	          .setMessage(vncCanvasActivity.getString(R.string.no_scale))
	          .setIcon(android.R.drawable.ic_dialog_info)
	          .setPositiveButton(android.R.string.ok, null)
	          .show();
			return false;
		}
	}
	/**
	 * Elimina cualquier zoom realizado a la imagen y la muestra en su escala original
	 * @param vncCanvas Lienzo al cual se ha realizado un zoom
	 * @param vncCanvasActivity Actividad que se esta ejecutando y que muestra el lienzo
	 * @author Alberto Poblacion Navarro
	 */
	public void reset(){
		//Se guardan los valores iniciales de la imagen para poder restaurarla.
		guardar_datos_iniciales();
		if(first_time == false)
		{
			new AlertDialog.Builder(vncCanvasActivity)
	        .setTitle(vncCanvasActivity.getString(R.string.reset))
	        .setMessage(vncCanvasActivity.getString(R.string.initial_pos))
	        .setIcon(android.R.drawable.ic_dialog_info)
	        .setPositiveButton(android.R.string.ok, new DialogInterface.OnClickListener(){
				@Override
				public void onClick(DialogInterface dialog, int which) {
						//Se restaura el  escalado, tamao y desplazamiento de la imagen
						zoom=1;
						matriz.setScale(zoom, zoom);
						vncCanvas.setImageMatrix(matriz);
						vncCanvas.setScaleType(ScaleType.CENTER);
						vncCanvas.setScaleType(ScaleType.MATRIX);
						vncCanvas.scrollTo(scrollXini, scrollYini);
						
						vncCanvas.bitmapData.framebufferwidth=ancho;
						vncCanvas.bitmapData.framebufferheight=alto;
						
						vncCanvasActivity.absoluteXPosition=absoluteX;
						vncCanvasActivity.absoluteYPosition=absoluteY;
						vncCanvas.warpMouse(vncCanvasActivity.absoluteXPosition+vncCanvas.getWidth(), vncCanvasActivity.absoluteYPosition+vncCanvas.getHeight());
						
						first_time=true;
				}
	        })
	        .setNegativeButton(android.R.string.cancel, null)
	        .show();
		}
	}
	/**
	 * Devuelve el valor que tenga actualmente el zoom para que solo pueda modificarse en Zoom.java
	 * @return valor de zoom
	 * @author Alberto Poblacion Navarro
	 */
	public float getZoom(){
		return zoom;
	}
	/**
	 * Devuelve la matriz que se aplique al escalado para que solo pueda modificarse en Zoom.java
	 * @return Matriz
	 * @author Alberto Poblacion Navarro
	 */
	static public Matrix getMatrix(){
		return matriz;
	}
	/**
	 * Se libera la instancia para que se cree una nueva la proxima vez. 
	 */
	static public void finalizar(){
			instancia = null;
	}
}

